searchNodes=[{"doc":"Specify allows you to make your configuration explicit: Specify exactly what fields are expected. Specify exactly what values these fields might take, by giving them parser-functions. Load their values from a slew of different locations, with &#39;explicitly passed in to the function&#39; as final option.","ref":"Specify.html","title":"Specify","type":"module"},{"doc":"Defines a configuration structure in the current module. Fields are added to this configuration structure by calling Specify.Schema.field/3 (which can be called just as field because Specify.Schema is autoamatically imported into the inner context of the call to defconfig.) The options that can be passed to this module are used as defaults for the options passed to a call to Specify.load/2 or YourModule.load/1. See also Specify.Schema.field/3 and Specify.Options Reflection The special function __specify__/1 will be defined on the module as well. It is not intended to be used by people that want to consume your configuration, but it is there to e.g. allow Specify.Provider implementations to be smarter in how they fetch the configuration for the module. For instance, configuration might be lazily fetched, when knowing what field names exist beforehand. YourModule.__specify__/1 supports the following publicly usable parameters: __specify__(:field_names) returns a MapSet of atoms, one per field in the configuration structure. __specify__(:defaults) returns a Map containing only the field_name =&gt; values of field names having default values. __specify__(:requireds) returns a MapSet of atoms, one per required field in the configuration structure. __specify__(:parsers) returns a Map of the format field_name =&gt; parser. __specify__(:field_options) returns a Map of the format field_name =&gt; options, where options is the keyword-list that was passed to the field macro.","ref":"Specify.html#defconfig/2","title":"Specify.defconfig/2","type":"macro"},{"doc":"Loads, parses, and normalizes the configuration of config_module, based on the current source settings, returning the result as a struct. (This is the more general way of calling config_module.load/1). See Specify.Options for more information of the options that can be supplied to this function, and how it can be configured further.","ref":"Specify.html#load/2","title":"Specify.load/2","type":"function"},{"doc":"Loads, parses and normalizes the configuration of config_module, using the provided explicit_values (and falling back to values configured elsewhere) This call is conceptually the same as Specify.load(config_module, [explicit_values: [] | options]), but makes it more explicit that values are meant to be passed in as arguments. Prefer this function if you do not intend to use Specify&#39;s &#39;cascading configuration&#39; functionality, such as when e.g. just parsing options passed to a function, use-statement or other macro.","ref":"Specify.html#load_explicit/3","title":"Specify.load_explicit/3","type":"function"},{"doc":"This struct represents the options you can pass to a call of Specify.load/2 (or YourModule.load/1). Metaconfiguration Besides making it nice and explicit to have the options listed here, Specify.Options has itself been defined using Specify.defconfig/2, which means that it (and thus what default options are passed on to to other Specify configurations) can be configured in the same way. Configuration structure documentation: This configuration was made using the Specify library. It contains the following fields: sources A list of structures that implement the Specify.Provider protocol, which will be used to fetch configuration from. Later entries in the list take precedence over earlier entries. Defaults always have the lowest precedence, and :explicit_values always have the highest precedence. A source can be: A struct. Example: %Specify.Provider.SystemEnv{}; A module that has a new/0-method which returns a struct. Example: Specify.Provider.SystemEnv; A tuple, whose first argument is a module and second argument is a map of arguments. This will be turned into a full-blown struct at startup using Kernel.struct/2. Example: {Specify.Provider.SystemEnv, %{prefix: &quot;CY&quot;, optional: true}}; A {module, function, arguments}-tuple, which will be called on startup. It should return a struct. Example: {Specify.Provider.SystemEnv, :new, [&quot;CY&quot;, [optional: true]]}. In all cases, the struct should implement the Specify.Provider protocol (and this is enforced at startup). Validated/parsed by calling Specify.Options.list_of_sources/1. Defaults to []. explicit_values A list or map (or other enumerable) representing explicit values that are to be used instead of what can be found in the implicit sources stack. Validated/parsed by calling Specify.Parsers.term/1. (Specified as :term) Defaults to []. missing_fields_error The error to be raised if a missing field which is required has been encountered. Validated/parsed by calling Specify.Parsers.term/1. (Specified as :term) Defaults to Specify.MissingRequiredFieldsError. parsing_error The error to be raised if a field value could not properly be parsed. Validated/parsed by calling Specify.Parsers.term/1. (Specified as :term) Defaults to Specify.ParsingError. explain When set to true, rather than returning the config struct, a map is returned with every field-key containing a list of consecutive found values. This is useful for debugging. Validated/parsed by calling Specify.Parsers.boolean/1. (Specified as :boolean) Defaults to false. Metaconfiguration Gotcha&#39;s Specify will only be able to find a source after it knows it exists. This means that it is impossible to define a different set of sources inside an external source. For this special case, Specify will look at the current process&#39; Process dictionary, falling back to the Application environment (also known as the Mix environment), and finally falling back to an empty list of sources (its default). So, from lowest to highest precedence, option values are loaded in this order: Specify.Options default Application Environment :specify Process Dictionary :specify field Options passed to Specify.defconfig Options passed to YourModule.load Requiring Specify to be configured in such an even more general way seems highly unlikely. If the current approach does turn out to not be good enough for your use-case, please open an issue on Specify&#39;s issue tracker.","ref":"Specify.Options.html","title":"Specify.Options","type":"module"},{"doc":"Loads, parses, and normalizes the configuration of Specify.Options, based on the current source settings, returning the result as a struct. For more information about the options this function supports, see Specify.load/2 and Specify.Options","ref":"Specify.Options.html#load/1","title":"Specify.Options.load/1","type":"function"},{"doc":"Loads, parses and normalizes the configuration of Specify.Options, using the provided explicit_values (and falling back to values configured elsewhere) For more information about the options this function supports, see Specify.load_explicit/3 and Specify.Options","ref":"Specify.Options.html#load_explicit/2","title":"Specify.Options.load_explicit/2","type":"function"},{"doc":"Simple functions to parse strings to datatypes commonly used during configuration. These functions can be used as parser/validator function in a call to Specify.Schema.field, by using their shorthand name (:integer as shorthand for &amp;Specify.Parsers.integer/1). (Of course, using their longhand name works as well.) Defining your own parser function A parser function receives the to-be-parsed/validated value as input, and should return {:ok, parsed_val} on success, or {:error, reason} on failure. Be aware that depending on where the configuration is loaded from, the to-be-parsed value might be a binary string, or already the Elixir type you want to convert it to.","ref":"Specify.Parsers.html","title":"Specify.Parsers","type":"module"},{"doc":"Parses an atom or a binary string representing an (existing) atom. Will not create new atoms (See String.to_existing_atom/1 for more info).","ref":"Specify.Parsers.html#atom/1","title":"Specify.Parsers.atom/1","type":"function"},{"doc":"Parses a boolean or a binary string representing a boolean value, turning it into a boolean.","ref":"Specify.Parsers.html#boolean/1","title":"Specify.Parsers.boolean/1","type":"function"},{"doc":"Parses a float and turns a binary string representing a float into an float. Will also accept integers, which are turned into their float equivalent.","ref":"Specify.Parsers.html#float/1","title":"Specify.Parsers.float/1","type":"function"},{"doc":"Parses a function. This can be a function capture, or a MFA (Module-Function-Arity) tuple, which will be transformed into the &amp;Module.function/arity capture. (So in either case, you end up with a function value that you can call using the dot operator, i.e. .() or .(maybe, some, args)). String Contexts For contexts in which values are specified as strings, the parser only supports the MFA format. This is for security (and ease of parsing) reasons.","ref":"Specify.Parsers.html#function/1","title":"Specify.Parsers.function/1","type":"function"},{"doc":"Parses an integer and turns binary string representing an integer into an integer.","ref":"Specify.Parsers.html#integer/1","title":"Specify.Parsers.integer/1","type":"function"},{"doc":"Parses a list of elements. In the case a binary string was passed, this parser uses Code.string_to_quoted under the hood to check for Elixir syntax, and will only accepts binaries representing lists. If a list was passed in (or after turning a binary into a list), it will try to parse each of the elements in turn.","ref":"Specify.Parsers.html#list/2","title":"Specify.Parsers.list/2","type":"function"},{"doc":"Parses a Module-Function-Arity tuple. Accepts it both as Elixir three-element tuple (where the first two elements are atoms, and the third is a nonnegative integer), or as string representation of the same. Will also check and ensure that this function is actually defined.","ref":"Specify.Parsers.html#mfa/1","title":"Specify.Parsers.mfa/1","type":"function"},{"doc":"Similar to float/1, but only accepts floats larger than or equal to 0.","ref":"Specify.Parsers.html#nonnegative_float/1","title":"Specify.Parsers.nonnegative_float/1","type":"function"},{"doc":"Similar to integer/1, but only accepts integers larger than or equal to 0.","ref":"Specify.Parsers.html#nonnegative_integer/1","title":"Specify.Parsers.nonnegative_integer/1","type":"function"},{"doc":"Similar to float/1, but only accepts floats larger than 0.","ref":"Specify.Parsers.html#positive_float/1","title":"Specify.Parsers.positive_float/1","type":"function"},{"doc":"Similar to integer/1, but only accepts integers larger than 0.","ref":"Specify.Parsers.html#positive_integer/1","title":"Specify.Parsers.positive_integer/1","type":"function"},{"doc":"Parses a binary string and turns anything that implements String.Chars into its binary string representation by calling to_string/1 on it.","ref":"Specify.Parsers.html#string/1","title":"Specify.Parsers.string/1","type":"function"},{"doc":"Accepts any Elixir term as-is. Will not do any parsing. Only use this as a last resort. It is usually better to create your own dedicated parsing function instead.","ref":"Specify.Parsers.html#term/1","title":"Specify.Parsers.term/1","type":"function"},{"doc":"Allows to pass in a &#39;timeout&#39; which is a common setting for OTP-related features, accepting either a positive integer, or the atom :infinity.","ref":"Specify.Parsers.html#timeout/1","title":"Specify.Parsers.timeout/1","type":"function"},{"doc":"","ref":"Specify.Parsers.html#unquote_atom/1","title":"Specify.Parsers.unquote_atom/1","type":"function"},{"doc":"Parses an atom or a binary string representing an (potentially not yet existing!) atom. Will create new atoms. Whenever possible, consider using atom/1 instead. (See String.to_atom/1 for more info on why creating new atoms is usually a bad idea).","ref":"Specify.Parsers.html#unsafe_atom/1","title":"Specify.Parsers.unsafe_atom/1","type":"function"},{"doc":"Protocol to load configuration from a signle source. Configuration Providers implement this protocol, which consists of only one function: load/2.","ref":"Specify.Provider.html","title":"Specify.Provider","type":"protocol"},{"doc":"Loads the configuration of specification module from the source indicated by struct. Its first argument is the implementation&#39;s own struct, the second argument being the configuration specification&#39;s module name. If extra information is required about the configuration specification to write a good implementation, the Reflection function module_name.__specify__ can be used to look these up. See also Specify.defconfig/2 and Specify.Options.","ref":"Specify.Provider.html#load/2","title":"Specify.Provider.load/2","type":"function"},{"doc":"","ref":"Specify.Provider.html#t:t/0","title":"Specify.Provider.t/0","type":"type"},{"doc":"A Configuration Provider source based on Mix.env() / Application.get_env/2. Examples The following examples use the following specification for reference: defmodule Elixir.Pet do require Specify Specify.defconfig do @doc &quot;The name of the pet&quot; field :name, :string @doc &quot;is it a dog or a cat?&quot; field :kind, :atom end end","ref":"Specify.Provider.MixEnv.html","title":"Specify.Provider.MixEnv","type":"module"},{"doc":"By default, will try to use Application.get_all_env(YourConfigModule) to fetch the source&#39;s configuration. A different application name can be used by supplying a different application argument. If the actual configuration is only inside one of the keys in this application, the second field key can also be provided. iex&gt; Application.put_env(Elixir.Pet, :name, &quot;Timmy&quot;) iex&gt; Application.put_env(Elixir.Pet, :kind, &quot;cat&quot;) iex&gt; Pet.load(sources: [Specify.Provider.MixEnv.new()]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Pet.load(sources: [Specify.Provider.MixEnv.new(Elixir.Pet)]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Application.put_env(:second_pet, :name, &quot;John&quot;) iex&gt; Application.put_env(:second_pet, :kind, :dog) iex&gt; Pet.load(sources: [Specify.Provider.MixEnv.new(:second_pet)]) %Pet{name: &quot;John&quot;, kind: :dog}","ref":"Specify.Provider.MixEnv.html#new/3","title":"Specify.Provider.MixEnv.new/3","type":"function"},{"doc":"A Configuration Provider source based on the current process&#39; Process Dictionary. Examples The following examples use the following specification for reference: defmodule Elixir.Pet do require Specify Specify.defconfig do @doc &quot;The name of the pet&quot; field :name, :string @doc &quot;is it a dog or a cat?&quot; field :kind, :atom end end","ref":"Specify.Provider.Process.html","title":"Specify.Provider.Process","type":"module"},{"doc":"By default, will try to use Process.get(YourModule) to fetch the source&#39;s configuration. A different key can be used by supplying a different key argument. iex&gt; Process.put(Pet, %{name: &quot;Timmy&quot;, kind: :cat}) iex&gt; Pet.load(sources: [Specify.Provider.Process.new(Pet)]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Process.put(:another_pet, %{name: &quot;John&quot;, kind: :dog}) iex&gt; Pet.load(sources: [Specify.Provider.Process.new(:another_pet)]) %Pet{name: &quot;John&quot;, kind: :dog}","ref":"Specify.Provider.Process.html#new/2","title":"Specify.Provider.Process.new/2","type":"function"},{"doc":"A Configuration Provider source based on System.get_env/2 Values will be loaded based on \#{prefix}_\#{capitalized_field_name}. prefix defaults to the capitalized name of the configuration specification module. capitalized_field_name is in CONSTANT_CASE (all-caps, with underscores as word separators). Examples The following examples use the following specification for reference: defmodule Elixir.Pet do require Specify Specify.defconfig do @doc &quot;The name of the pet&quot; field :name, :string @doc &quot;is it a dog or a cat?&quot; field :kind, :atom, system_env_name: &quot;TYPE&quot; end end Note that if a field has a different name than the environment variable you want to read from, you can add the system_env_name: option when specifying the field, as has been done for the :kind field in the example module above. iex&gt; System.put_env(&quot;PET_NAME&quot;, &quot;Timmy&quot;) iex&gt; System.put_env(&quot;PET_TYPE&quot;, &quot;cat&quot;) iex&gt; Pet.load(sources: [Specify.Provider.SystemEnv.new()]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; Pet.load(sources: [Specify.Provider.SystemEnv.new(&quot;PET&quot;)]) %Pet{name: &quot;Timmy&quot;, kind: :cat} iex&gt; System.put_env(&quot;SECOND_PET_NAME&quot;, &quot;John&quot;) iex&gt; System.put_env(&quot;SECOND_PET_TYPE&quot;, &quot;dog&quot;) iex&gt; Pet.load(sources: [Specify.Provider.SystemEnv.new(&quot;SECOND_PET&quot;)]) %Pet{name: &quot;John&quot;, kind: :dog}","ref":"Specify.Provider.SystemEnv.html","title":"Specify.Provider.SystemEnv","type":"module"},{"doc":"","ref":"Specify.Provider.SystemEnv.html#new/2","title":"Specify.Provider.SystemEnv.new/2","type":"function"},{"doc":"Functions that can be used inside Specify.defconfig/2.","ref":"Specify.Schema.html","title":"Specify.Schema","type":"module"},{"doc":"Specifies a field that is part of the configuration struct. Can/should only be called inside a call to Specify.defconfig. name should be an atom representing the field. It will also become the field name for the struct that is created. parser should either be: an arity-one function capture like &amp;YourModule.some_type_parser/1. An atom representing one of the common parser function names in Specify.Parsers like :integer, :string, :boolean or :term. A two-element tuple like {:list, :atom}. The first element represents the &#39;collection parser&#39; which is an arity-2 function that takes the &#39;element parser&#39; as second argument. The second element is the &#39;element parser&#39;. Both of the elements listed in the tuple can also be either an atom, or a function capture with the correct arity. (like {&amp;YourAwesomeModule.fancy_collection/2, :integer}). parser defaults to :string. Supported field options are: default:, supplies a default value to this field. If not set, the configuration field is set to be required. You are highly encouraged to add a @documentation text above each and every field; these will be added to the configuration&#39;s module documentation.","ref":"Specify.Schema.html#field/3","title":"Specify.Schema.field/3","type":"macro"},{"doc":"Default exception to be raised when a required field is not existent in any configuration source.","ref":"Specify.MissingRequiredFieldsError.html","title":"Specify.MissingRequiredFieldsError","type":"exception"},{"doc":"Default exception to be raised when it is impossible to parse one of the configuration values. (See also Specify.Parsers)","ref":"Specify.ParsingError.html","title":"Specify.ParsingError","type":"exception"},{"doc":"Specify is a library to create Comfortable, Explicit, Multi-Layered and Well-Documented Specifications for all your configurations, settings and options in Elixir. Basic features: Configuration is converted to a struct, with fields being parsed to their appropriate types. Specify a stack of sources to fetch the configuration from. Always possible to override local configuration using plain arguments to a function call. Fail-fast on missing or malformed values. Auto-generated documentation based on your config specification. Specify can be used both to create normalized configuration structs during runtime and compile-time using both implicit external configuration sources and explicit arguments to a function call.","ref":"readme.html","title":"README","type":"extras"},{"doc":"You can install Specify by adding specify to your list of dependencies in mix.exs: def deps do [ {:specify, &quot;~&gt; 0.7.0&quot;} ] end Documentation can be found at https://hexdocs.pm/specify.","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"Basic usage is as follows, using Specify.defconfig/1: defmodule Cosette.CastleOnACloud do require Specify Specify.defconfig do @doc &quot;there are no floors for me to sweep&quot; field :floors_to_sweep, :integer, default: 0 @doc &quot;there are a hundred boys and girls&quot; field :amount_boys_and_girls, :integer, default: 100 @doc &quot;The lady all in white holds me and sings a lullaby&quot; field :lullaby, :string @doc &quot;Crying is usually not allowed&quot; field :crying_allowed, :boolean, default: false end end and later Specify.load/2, Specify.load_explicit/3 (or YourModule.load/1, YourModule.load_explicit/2 which are automatically defined). iex&gt; Cosette.CastleOnACloud.load(explicit_values: [lullaby: &quot;I love you very much&quot;, crying_allowed: true]) %Cosette.CastleOnACloud{ crying_allowed: true, floors_to_sweep: 0, lullaby: &quot;I love you very much&quot;, amount_boys_and_girls: 100 } Mandatory Fields Notice that since the :lullaby-field is mandatory, if it is not defined in any of the configuration sources, an error will be thrown: Cosette.CastleOnACloud.load ** (Specify.MissingRequiredFieldsError) Missing required fields for `Elixir.Cosette.CastleOnACloud`: `:lullaby`. (specify) lib/specify.ex:179: Specify.prevent_missing_required_fields!/3 (specify) lib/specify.ex:147: Specify.load/2 Loading from Sources Loading from another source is easy: iex&gt; Application.put_env(Cosette.CastleOnACloud, :lullaby, &quot;sleep little darling&quot;) # or: in a Mix config.ex file config Cosette.CastleOnACloud, lullaby: &quot;sleep little darling&quot; iex&gt; Cosette.CastleOnACloud.load(sources: [Specify.Provider.MixEnv]) %Cosette.CastleOnACloud{ crying_allowed: false, floors_to_sweep: 0, lullaby: &quot;sleep little darling&quot;, no_boys_and_girls: 100 } Rather than passing in the sources when loading the configuration, it often makes more sense to specify them when defining the configuration: defmodule Cosette.CastleOnACloud do require Specify Specify.defconfig sources: [Specify.Provider.MixEnv] do # ... end end","ref":"readme.html#examples","title":"README - Examples","type":"extras"},{"doc":"Providers can be specified by passing them to the sources: option (while loading the configuration structure or while defining it). They can also be set globally by altering the sources: key of the Specify application environment, or per-process using the :sources subkey of the Specify key in the current process&#39; dictionary (Process.put_env). Be aware that for bootstrapping reasons, it is impossible to override the :sources field globally in an external source (because Specify would not know where to find it). Specify comes with the following built-in providers: Specify.Provider.MixEnv, which uses Mix.env / Application.get_env to read from the application environment. Specify.Provider.SystemEnv, which uses System.get_env to read from system environment variables. Specify.Provider.Process, which uses Process.get to read from the current process&#39; dictionary. Often, Providers have sensible default values on how they work, making their usage simpler: Specify.Provider.Process will look at the configured key, but will default to the configuration specification module name. Specify.Provider.MixEnv will look at the configured application_name and key, but will default to the whole environment of an application (Application.get_all_env) if no key was set, with application_name defaulting to the configuration specification module name. Specify.Provider.SystemEnv will look at the configured prefix but will default to the module name (in all caps), followed by the field name (in all caps, separated by underscores). What names should be used for a field is also configurable.","ref":"readme.html#providers","title":"README - Providers","type":"extras"},{"doc":"Providers implement the Specify.Provider protocol, which consists of only one function: load/2. Its first argument is the implementation&#39;s own struct, the second argument being the configuration specification&#39;s module name. If extra information is required about the configuration specification to write a good implementation, the Reflection function module_name.__specify__ can be used to look these up.","ref":"readme.html#writing-providers","title":"README - Writing Providers","type":"extras"},{"doc":"[x] Compound parsers for collections using {collection_parser, element_parser}-syntax, with provided :list parser. [x] Main functionality documentation. [x] Parsers documentation. [x] Writing basic Tests [x] Specify.Parsers [x] Main Specify module and functionality. [x] Thinking on how to handle environment variable names (capitalization, prefixes). [x] Environment Variables (System.get_env) provider [x] Specify Provider Tests. [ ] Better/more examples [ ] Stable release","ref":"readme.html#roadmap","title":"README - Roadmap","type":"extras"},{"doc":"(D)ETS provider CLI arguments provider, which could be helpful for defining e.g. Mix tasks. .env files provider. JSON and YML files provider. Nested configs? Possibility to load without raising on parsing falure (instead returning a success/failure tuple?) Watching for updates and call a configurable handler function when configuration has changed.","ref":"readme.html#possibilities-for-the-future","title":"README - Possibilities for the future","type":"extras"},{"doc":"0.7.2 - Makes functions clickable in the generated documentation. Thank you, @tanguilp! 0.7.1 - Pretty-prints long default values in a multi-line code block in the documentation (#2). 0.7 - Adds an optional key to the built-in providers. They will only return {error, :not_found} if they are not set to optional. Also adds two new ways to indicate sources, which are helpful in environments where you do not have access to the structs directly (such as Mix.Config or the newer Elixir.Config files.) 0.6 - Adds the mfa and function builtin parsers. 0.5 - Adds the nonnegative_integer, positive_integer, nonnegative_float, positive_float and timeout builtin parsers. 0.4.5 - Fixes built-in integer and float parsers to not crash on input like &quot;10a&quot; (but instead return {:error, _}). 0.4.4 - Fixes references to validation/parsing functions in documentation. 0.4.2 - Finishes provider tests; bugfix for the MixEnv provider. 0.4.1 - Improves documentation. 0.4.0 - Name change: from &#39;Confy&#39; to &#39;Specify&#39;. This name has been chosen to be more clear about the intent of the library. 0.3.0 - Changed overrides: to explicit_values: and added Specify.load_explicit/3 function. (Also added tests and fixed parser bugs). 0.2.0 - Initially released version","ref":"readme.html#changelog","title":"README - Changelog","type":"extras"},{"doc":"I want to thank Chris Keathley for his interesting library Vapor which helped inspire Specify. I also want to thank José Valim for the great conversations we&#39;ve had about the advantages and disadvantages of various approaches to configuring Elixir applications.","ref":"readme.html#attribution","title":"README - Attribution","type":"extras"}]